/*****************************************************************************
 * HClassMLP.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HClassMlp
 *
 * (c) 1996-2003 by MVTec Software GmbH
 *                  www.mvtec.com
 * 
 *****************************************************************************
 *
 * $Revision: 1.2 $
 * $Date: 2003/10/02 09:53:12 $
 *
 */

#ifndef H_CLASS_MLP_H
#define H_CLASS_MLP_H

class LIntExport HClassMlp: public HRootObject {
public:
  HClassMlp(void) {id = -1;}
  HClassMlp(const HTuple &NumInput, const HTuple &NumHidden,
            const HTuple &NumOutput, const HTuple &OutputFunction,
            const HTuple &Preprocessing, const HTuple &NumComponents,
            const HTuple &RandSeed);

  ~HClassMlp(void);
private:
  // Copying and Assignment not allowed
  HClassMlp(const HClassMlp &inst);
  HClassMlp &operator= (const HClassMlp &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HClassMlp";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  // Handle
  long  id;
public:
  // Read a multilayer perceptron from a file.
  virtual void ReadClassMlp(const HTuple &FileName);
  virtual void ReadClassMlp(const char *FileName);
  // Write a multilayer perceptron to a file.
  virtual void WriteClassMlp(const HTuple &FileName) const;
  virtual void WriteClassMlp(const char *FileName) const;
  // Read the training data of a multilayer perceptron from a file.
  virtual void ReadSamplesClassMlp(const HTuple &FileName) const;
  virtual void ReadSamplesClassMlp(const char *FileName) const;
  // Write the training data of a multilayer perceptron to a file.
  virtual void WriteSamplesClassMlp(const HTuple &FileName) const;
  virtual void WriteSamplesClassMlp(const char *FileName) const;
  // Calculate the class of a feature vector by a multilayer perceptron.
  virtual HTuple ClassifyClassMlp(const HTuple &Features, const HTuple &Num, HTuple *Confidence) const;
  virtual long ClassifyClassMlp(const HTuple &Features, const HTuple &Num, double *Confidence) const;
  // Calculate the evaluation of a feature vector by a multilayer
  // perceptron.
  virtual HTuple EvaluateClassMlp(const HTuple &Features) const;
  // Train a multilayer perceptron.
  virtual double TrainClassMlp(const HTuple &MaxIterations, const HTuple &WeightTolerance, const HTuple &ErrorTolerance, HTuple *ErrorLog) const;
  virtual double TrainClassMlp(long MaxIterations, double WeightTolerance, double ErrorTolerance, HTuple *ErrorLog) const;
  // Compute the information content of the preprocessed feature vectors
  // of a multilayer perceptron.
  virtual HTuple GetPrepInfoClassMlp(const HTuple &Preprocessing, HTuple *CumInformationCont) const;
  virtual HTuple GetPrepInfoClassMlp(const char *Preprocessing, HTuple *CumInformationCont) const;
  // Return the number of training samples stored in the training data of
  // a multilayer perceptron.
  virtual long GetSampleNumClassMlp() const;
  // Return a training sample from the training data of a multilayer
  // perceptron.
  virtual HTuple GetSampleClassMlp(const HTuple &NumSample, HTuple *Target) const;
  // Add a training sample to the training data of a multilayer
  // perceptron.
  virtual void AddSampleClassMlp(const HTuple &Features, const HTuple &Target) const;
  // Return the parameters of a multilayer perceptron.
  virtual long GetParamsClassMlp(HTuple *NumHidden, HTuple *NumOutput, HTuple *OutputFunction, HTuple *Preprocessing, HTuple *NumComponents) const;
  virtual long GetParamsClassMlp(long *NumHidden, long *NumOutput, char *OutputFunction, char *Preprocessing, long *NumComponents) const;
  // Create a multilayer perceptron for classification or regression.
  virtual void CreateClassMlp(const HTuple &NumInput, const HTuple &NumHidden, const HTuple &NumOutput, const HTuple &OutputFunction, const HTuple &Preprocessing, const HTuple &NumComponents, const HTuple &RandSeed);
  virtual void CreateClassMlp(long NumInput, long NumHidden, long NumOutput, const char *OutputFunction, const char *Preprocessing, long NumComponents, long RandSeed);
  // Classify an image with a multilayer perceptron.
  virtual HRegionArray ClassifyImageClassMlp(const HImage &Image, const HTuple &RejectionThreshold) const;
  virtual HRegionArray ClassifyImageClassMlp(const HImage &Image, double RejectionThreshold) const;
  // Add training samples from an image to the training data of a
  // multilayer perceptron.
  virtual void AddSamplesImageClassMlp(const HImage &Image, const HRegionArray &ClassRegions) const;
};

#endif
