/*****************************************************************************
 * HRegionArray.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Types for regions arrays
 *
 * (c) 1996-2000 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.12 $
 * $Date: 2000/07/25 08:17:53 $
 *
 */


#ifndef H_REGION_ARRAY_H
#define H_REGION_ARRAY_H

//#include "Hconst.h"
//#include "IPtype.h"


/****************************************************************************/
/*                            HRegionArray                                  */
/****************************************************************************/
class LIntExport HRegionArray: public HObjectArray {
public:
  HRegionArray(void) : array(NULL) {num_region_arrays++;}
  HRegionArray(Hkey *key, long num);
  HRegionArray(const Hobject &obj);
  HRegionArray(const HRegion &reg);
  HRegionArray(const HRegionArray &arr);
  ~HRegionArray(void);
  HRegionArray &operator = (const HRegionArray &arr);
  
  // Name der Halcon-Klasse -> get_obj_class
  virtual const char *HClassName(void)    const;
  virtual const char *ClassName(void)     const;
  virtual       INT4  ClassDBID(void)     const {return REGION_ID;}
  virtual       Hkey  Key(long i)         const {return array[i].Key();}
  virtual    Hobject  ElemId(long i)      const {return array[i].Id();}
  
  // Zugriff und Manipulation
  HRegion      const &operator [] (long index) const;
  HRegion            &operator [] (long index);
  HRegionArray        operator () (long min, long max) const;
  HRegionArray       &Append(const HRegion &reg);
  HRegionArray       &Append(const HRegionArray &reg);
  virtual void        SetKeys(Hkey *key, long num);
  virtual void        Display(const HWindow &w) const;
  virtual void        DeleteArray(void);
  
  // Infix-Operationen
  HRegionArray operator >> (double radius) const;
  HRegionArray operator << (double radius) const;
  HRegionArray operator + (const HRegion &reg) const;
  HRegionArray operator - (const HRegion &reg) const;
  HRegionArray operator ~ (void) const;
  HRegionArray operator & (const HRegionArray &reg) const;
  HRegionArray operator | (const HRegionArray &reg) const;
  HRegionArray operator / (const HRegionArray &reg) const;
  
  // Klassenfunktionen
  static long NumInst(void) { return num_region_arrays; }
private:
  static long     num_region_arrays; 
  HRegion  *array;
  void     NewLength(long new_num, long old_num);
  void     CopyHRegionArray(const HRegionArray &in);
public:
  static HRegionArray GrabRegionAsync(const HTuple &FGHandle, const HTuple &MaxDelay);
  static HRegionArray GrabRegionAsync(long FGHandle, double MaxDelay);
  static HRegionArray GrabRegion(const HTuple &FGHandle);
  static HRegionArray GrabRegion(long FGHandle);
  virtual HImageArray AddChannels(const HImage &Image) const;
  virtual void WriteRegion(const HTuple &FileName) const;
  virtual void WriteRegion(const char *FileName) const;
  static HRegionArray ReadRegion(const HTuple &FileName);
  static HRegionArray ReadRegion(const char *FileName);
  virtual HRegionArray CloseEdgesLength(const HImage &Gradient, const HTuple &MinAmplitude, const HTuple &MaxGapLength) const;
  virtual HRegionArray CloseEdgesLength(const HImage &Gradient, long MinAmplitude, long MaxGapLength) const;
  virtual HRegionArray CloseEdges(const HImage &EdgeImage, const HTuple &MinAmplitude) const;
  virtual HRegionArray CloseEdges(const HImage &EdgeImage, long MinAmplitude) const;
  virtual HTuple NoiseDistributionMean(const HImage &Image, const HTuple &FilterSize) const;
  virtual HTuple NoiseDistributionMean(const HImage &Image, long FilterSize) const;
  virtual HTuple FuzzyEntropy(const HImage &Image, const HTuple &Apar, const HTuple &Cpar) const;
  virtual HTuple FuzzyEntropy(const HImage &Image, long Apar, long Cpar) const;
  virtual HTuple FuzzyPerimeter(const HImage &Image, const HTuple &Apar, const HTuple &Cpar) const;
  virtual HTuple FuzzyPerimeter(const HImage &Image, long Apar, long Cpar) const;
  virtual HImage RegionToMean(const HImage &Image) const;
  virtual void DispRegion(const HTuple &WindowHandle) const;
  virtual void DispRegion(long WindowHandle) const;
  // Interactive movement of a region with restriction of positions.
  virtual HRegionArray DragRegion3(const HRegionArray &MaskRegion, const HTuple &WindowHandle, const HTuple &Row, const HTuple &Column) const;
  virtual HRegionArray DragRegion3(const HRegionArray &MaskRegion, long WindowHandle, long Row, long Column) const;
  // Interactive movement of a region with fixpoint specification.
  virtual HRegionArray DragRegion2(const HTuple &WindowHandle, const HTuple &Row, const HTuple &Column) const;
  virtual HRegionArray DragRegion2(long WindowHandle, long Row, long Column) const;
  // Interactive moving of a region.
  virtual HRegionArray DragRegion1(const HTuple &WindowHandle) const;
  virtual HRegionArray DragRegion1(long WindowHandle) const;
  virtual HTuple FitSurfaceFirstOrder(const HImage &Image, const HTuple &Algorithm, const HTuple &Iterations, const HTuple &ClippingFactor, HTuple *Beta, HTuple *Gamma) const;
  virtual HTuple FitSurfaceFirstOrder(const HImage &Image, const char *Algorithm, long Iterations, double ClippingFactor, HTuple *Beta, HTuple *Gamma) const;
  virtual HTuple FitSurfaceSecondOrder(const HImage &Image, const HTuple &Algorithm, const HTuple &Iterations, const HTuple &ClippingFactor, HTuple *Beta, HTuple *Gamma, HTuple *Delta, HTuple *Epsilon, HTuple *Zeta) const;
  virtual HTuple FitSurfaceSecondOrder(const HImage &Image, const char *Algorithm, long Iterations, double ClippingFactor, HTuple *Beta, HTuple *Gamma, HTuple *Delta, HTuple *Epsilon, HTuple *Zeta) const;
  // Select regions based on gray value features.
  virtual HRegionArray SelectGray(const HImage &Image, const HTuple &Features, const HTuple &Operation, const HTuple &Min, const HTuple &Max) const;
  virtual HRegionArray SelectGray(const HImage &Image, const char *Features, const char *Operation, double Min, double Max) const;
  virtual HTuple MinMaxGray(const HImage &Image, const HTuple &Percent, HTuple *Max, HTuple *Range) const;
  virtual HTuple MinMaxGray(const HImage &Image, double Percent, HTuple *Max, HTuple *Range) const;
  virtual HTuple Intensity(const HImage &Image, HTuple *Deviation) const;
  virtual HImage Histo2dim(const HImage &ImageCol, const HImage &ImageRow) const;
  virtual HTuple GrayHistoAbs(const HImage &Image, const HTuple &Quantization) const;
  virtual HTuple GrayHistoAbs(const HImage &Image, double Quantization) const;
  virtual HTuple GrayHisto(const HImage &Image, HTuple *RelativeHisto) const;
  virtual HTuple EntropyGray(const HImage &Image, HTuple *Anisotropy) const;
  virtual HTuple CoocFeatureImage(const HImage &Image, const HTuple &LdGray, const HTuple &Direction, HTuple *Correlation, HTuple *Homogeneity, HTuple *Contrast) const;
  virtual HTuple CoocFeatureImage(const HImage &Image, long LdGray, long Direction, HTuple *Correlation, HTuple *Homogeneity, HTuple *Contrast) const;
  virtual HImageArray GenCoocMatrix(const HImage &Image, const HTuple &LdGray, const HTuple &Direction) const;
  virtual HImageArray GenCoocMatrix(const HImage &Image, long LdGray, long Direction) const;
  virtual HTuple MomentsGrayPlane(const HImage &Image, HTuple *MCol, HTuple *Alpha, HTuple *Beta, HTuple *Mean) const;
  virtual HTuple PlaneDeviation(const HImage &Image) const;
  virtual HTuple EllipticAxisGray(const HImage &Image, HTuple *Rb, HTuple *Phi) const;
  virtual HTuple AreaCenterGray(const HImage &Image, HTuple *Row, HTuple *Column) const;
  virtual HTuple MomentsRegionCentralInvar(HTuple *PSI2, HTuple *PSI3, HTuple *PSI4) const;
  virtual HTuple MomentsRegionCentral(HTuple *I2, HTuple *I3, HTuple *I4) const;
  virtual HTuple MomentsRegion3rdInvar(HTuple *M12, HTuple *M03, HTuple *M30) const;
  virtual HTuple MomentsRegion3rd(HTuple *M12, HTuple *M03, HTuple *M30) const;
  virtual HTuple SmallestRectangle2(HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2) const;
  virtual HTuple SmallestRectangle1(HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  virtual HTuple SmallestCircle(HTuple *Column, HTuple *Radius) const;
  virtual HRegionArray SelectShapeProto(const HRegionArray &Pattern, const HTuple &Feature, const HTuple &Min, const HTuple &Max) const;
  virtual HRegionArray SelectShapeProto(const HRegionArray &Pattern, const char *Feature, double Min, double Max) const;
  // Choose regions with the aid of shape features.
  virtual HRegionArray SelectShape(const HTuple &Features, const HTuple &Operation, const HTuple &Min, const HTuple &Max) const;
  virtual HRegionArray SelectShape(const char *Features, const char *Operation, double Min, double Max) const;
  virtual HTuple RunlengthFeatures(HTuple *KFactor, HTuple *LFactor, HTuple *MeanLength, HTuple *Bytes) const;
  virtual HTuple FindNeighbors(const HRegionArray &Regions2, const HTuple &MaxDistance, HTuple *RegionIndex2) const;
  virtual HTuple FindNeighbors(const HRegionArray &Regions2, long MaxDistance, HTuple *RegionIndex2) const;
  virtual HTuple MomentsRegion2ndRelInvar(HTuple *PHI2) const;
  virtual HTuple MomentsRegion2ndInvar(HTuple *M20, HTuple *M02) const;
  virtual HTuple MomentsRegion2nd(HTuple *M20, HTuple *M02, HTuple *Ia, HTuple *Ib) const;
  virtual HTuple DistanceRrMin(const HRegionArray &Regions2, HTuple *Row1, HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  virtual HTuple DistanceRrMinDil(const HRegionArray &Regions2) const;
  virtual HTuple DiameterRegion(HTuple *Column1, HTuple *Row2, HTuple *Column2, HTuple *Diameter) const;
  virtual HBool TestRegionPoint(const HTuple &Row, const HTuple &Column) const;
  virtual HBool TestRegionPoint(long Row, long Column) const;
  // Index of all regions containing a given pixel.
  virtual HTuple GetRegionIndex(const HTuple &Row, const HTuple &Column) const;
  virtual HTuple GetRegionIndex(long Row, long Column) const;
  // Choose all regions containing a given pixel.
  virtual HRegionArray SelectRegionPoint(const HTuple &Row, const HTuple &Column) const;
  virtual HRegionArray SelectRegionPoint(long Row, long Column) const;
  virtual HRegionArray SelectShapeStd(const HTuple &Shape, const HTuple &Percent) const;
  virtual HRegionArray SelectShapeStd(const char *Shape, double Percent) const;
  virtual HTuple HammingDistanceNorm(const HRegionArray &Regions2, const HTuple &Norm, HTuple *Similarity) const;
  virtual HTuple HammingDistanceNorm(const HRegionArray &Regions2, const char *Norm, HTuple *Similarity) const;
  virtual HTuple HammingDistance(const HRegionArray &Regions2, HTuple *Similarity) const;
  virtual HTuple Eccentricity(HTuple *Bulkiness, HTuple *StructureFactor) const;
  virtual HTuple EulerNumber(void) const;
  virtual HTuple OrientationRegion(void) const;
  virtual HTuple EllipticAxis(HTuple *Rb, HTuple *Phi) const;
  virtual HTuple SelectRegionSpatial(const HRegionArray &Regions2, const HTuple &Direction, HTuple *RegionIndex2) const;
  virtual HTuple SelectRegionSpatial(const HRegionArray &Regions2, const char *Direction, HTuple *RegionIndex2) const;
  virtual HTuple SpatialRelation(const HRegionArray &Regions2, const HTuple &Percent, HTuple *RegionIndex2, HTuple *Relation1, HTuple *Relation2) const;
  virtual HTuple SpatialRelation(const HRegionArray &Regions2, long Percent, HTuple *RegionIndex2, HTuple *Relation1, HTuple *Relation2) const;
  virtual HTuple Convexity(void) const;
  virtual HTuple Contlength(void) const;
  virtual HTuple ConnectAndHoles(HTuple *NumHoles) const;
  virtual HTuple Compactness(void) const;
  virtual HTuple Circularity(void) const;
  virtual HTuple AreaCenter(HTuple *Row, HTuple *Column) const;
  virtual HTuple Roundness(HTuple *Sigma, HTuple *Roundness, HTuple *Sides) const;
  virtual HTuple InnerCircle(HTuple *Column, HTuple *Radius) const;
  virtual HRegionArray Pruning(const HTuple &Length) const;
  virtual HRegionArray Pruning(long Length) const;
  virtual HRegionArray Boundary(const HTuple &BoundaryType) const;
  virtual HRegionArray Boundary(const char *BoundaryType) const;
  virtual HRegionArray Fitting(const HRegionArray &StructElements) const;
  static HRegionArray GenStructElements(const HTuple &Type, const HTuple &Row, const HTuple &Column);
  static HRegionArray GenStructElements(const char *Type, long Row, long Column);
  virtual HRegionArray TransposeRegion(const HTuple &Row, const HTuple &Column) const;
  virtual HRegionArray TransposeRegion(long Row, long Column) const;
  virtual HRegionArray ThinningSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegionArray ThinningSeq(const char *GolayElement, long Iterations) const;
  virtual HRegionArray ThinningGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegionArray ThinningGolay(const char *GolayElement, long Rotation) const;
  virtual HRegionArray Thinning(const HRegion &StructElement1, const HRegion &StructElement2, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegionArray Thinning(const HRegion &StructElement1, const HRegion &StructElement2, long Row, long Column, long Iterations) const;
  virtual HRegionArray ThickeningSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegionArray ThickeningSeq(const char *GolayElement, long Iterations) const;
  virtual HRegionArray ThickeningGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegionArray ThickeningGolay(const char *GolayElement, long Rotation) const;
  virtual HRegionArray Thickening(const HRegion &StructElement1, const HRegion &StructElement2, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegionArray Thickening(const HRegion &StructElement1, const HRegion &StructElement2, long Row, long Column, long Iterations) const;
  virtual HRegionArray HitOrMissSeq(const HTuple &GolayElement) const;
  virtual HRegionArray HitOrMissSeq(const char *GolayElement) const;
  virtual HRegionArray HitOrMissGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegionArray HitOrMissGolay(const char *GolayElement, long Rotation) const;
  virtual HRegionArray HitOrMiss(const HRegion &StructElement1, const HRegion &StructElement2, const HTuple &Row, const HTuple &Column) const;
  virtual HRegionArray HitOrMiss(const HRegion &StructElement1, const HRegion &StructElement2, long Row, long Column) const;
  virtual HRegionArray MorphSkiz(const HTuple &Iterations1, const HTuple &Iterations2) const;
  virtual HRegionArray MorphSkiz(long Iterations1, long Iterations2) const;
  virtual HRegionArray MorphSkeleton(void) const;
  virtual HRegionArray MorphHat(const HRegion &StructElement) const;
  virtual HRegionArray BottomHat(const HRegion &StructElement) const;
  virtual HRegionArray TopHat(const HRegion &StructElement) const;
  virtual HRegionArray MinkowskiSub2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegionArray MinkowskiSub2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  virtual HRegionArray MinkowskiSub1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegionArray MinkowskiSub1(const HRegion &StructElement, long Iterations) const;
  virtual HRegionArray MinkowskiAdd2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegionArray MinkowskiAdd2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  virtual HRegionArray MinkowskiAdd1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegionArray MinkowskiAdd1(const HRegion &StructElement, long Iterations) const;
  virtual HRegionArray ClosingRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegionArray ClosingRectangle1(long Width, long Height) const;
  virtual HRegionArray ClosingGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegionArray ClosingGolay(const char *GolayElement, long Rotation) const;
  virtual HRegionArray ClosingCircle(const HTuple &Radius) const;
  virtual HRegionArray ClosingCircle(double Radius) const;
  virtual HRegionArray Closing(const HRegion &StructElement) const;
  virtual HRegionArray OpeningSeg(const HRegion &StructElement) const;
  virtual HRegionArray OpeningGolay(const HTuple &GolayElement, const HTuple &Rotation) const;
  virtual HRegionArray OpeningGolay(const char *GolayElement, long Rotation) const;
  virtual HRegionArray OpeningRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegionArray OpeningRectangle1(long Width, long Height) const;
  virtual HRegionArray OpeningCircle(const HTuple &Radius) const;
  virtual HRegionArray OpeningCircle(double Radius) const;
  virtual HRegionArray Opening(const HRegion &StructElement) const;
  virtual HRegionArray ErosionSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegionArray ErosionSeq(const char *GolayElement, long Iterations) const;
  virtual HRegionArray ErosionGolay(const HTuple &GolayElement, const HTuple &Iterations, const HTuple &Rotation) const;
  virtual HRegionArray ErosionGolay(const char *GolayElement, long Iterations, long Rotation) const;
  virtual HRegionArray ErosionRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegionArray ErosionRectangle1(long Width, long Height) const;
  virtual HRegionArray ErosionCircle(const HTuple &Radius) const;
  virtual HRegionArray ErosionCircle(double Radius) const;
  virtual HRegionArray Erosion2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegionArray Erosion2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  virtual HRegionArray Erosion1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegionArray Erosion1(const HRegion &StructElement, long Iterations) const;
  virtual HRegionArray DilationSeq(const HTuple &GolayElement, const HTuple &Iterations) const;
  virtual HRegionArray DilationSeq(const char *GolayElement, long Iterations) const;
  virtual HRegionArray DilationGolay(const HTuple &GolayElement, const HTuple &Iterations, const HTuple &Rotation) const;
  virtual HRegionArray DilationGolay(const char *GolayElement, long Iterations, long Rotation) const;
  virtual HRegionArray DilationRectangle1(const HTuple &Width, const HTuple &Height) const;
  virtual HRegionArray DilationRectangle1(long Width, long Height) const;
  virtual HRegionArray DilationCircle(const HTuple &Radius) const;
  virtual HRegionArray DilationCircle(double Radius) const;
  virtual HRegionArray Dilation2(const HRegion &StructElement, const HTuple &Row, const HTuple &Column, const HTuple &Iterations) const;
  virtual HRegionArray Dilation2(const HRegion &StructElement, long Row, long Column, long Iterations) const;
  virtual HRegionArray Dilation1(const HRegion &StructElement, const HTuple &Iterations) const;
  virtual HRegionArray Dilation1(const HRegion &StructElement, long Iterations) const;
  virtual HTuple DoOcrMulti(const HImage &Image, const HTuple &OcrHandle, HTuple *Confidence) const;
  virtual HTuple DoOcrMulti(const HImage &Image, long OcrHandle, HTuple *Confidence) const;
  virtual double TraindOcrClassBox(const HImage &Image, const HTuple &OcrHandle, const HTuple &Class) const;
  virtual double TraindOcrClassBox(const HImage &Image, long OcrHandle, const char *Class) const;
  virtual void WriteOcrTrainf(const HImage &Image, const HTuple &Class, const HTuple &FileName) const;
  virtual void WriteOcrTrainf(const HImage &Image, const char *Class, const char *FileName) const;
  // Sorting of regions with respect to their relative position.
  virtual HRegionArray SortRegion(const HTuple &SortMode, const HTuple &Order, const HTuple &RowOrCol) const;
  virtual HRegionArray SortRegion(const char *SortMode, const char *Order, const char *RowOrCol) const;
  virtual HTuple TestdOcrClassBox(const HImage &Image, const HTuple &OcrHandle, const HTuple &Class) const;
  virtual HTuple TestdOcrClassBox(const HImage &Image, long OcrHandle, const char *Class) const;
  virtual void AppendOcrTrainf(const HImage &Image, const HTuple &Class, const HTuple &FileName) const;
  virtual void AppendOcrTrainf(const HImage &Image, const char *Class, const char *FileName) const;
  static HRegionArray GenRectangle2(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Length1, const HTuple &Length2);
  static HRegionArray GenRectangle2(double Row, double Column, double Phi, double Length1, double Length2);
  static HRegionArray GenRectangle1(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2);
  static HRegionArray GenRectangle1(double Row1, double Column1, double Row2, double Column2);
  static HRegionArray GenEllipse(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2);
  static HRegionArray GenEllipse(double Row, double Column, double Phi, double Radius1, double Radius2);
  static HRegionArray GenCircle(const HTuple &Row, const HTuple &Column, const HTuple &Radius);
  static HRegionArray GenCircle(double Row, double Column, double Radius);
  // Create random regions like circles, rectangles and ellipses.
  static HRegionArray GenRandomRegions(const HTuple &Type, const HTuple &WidthMin, const HTuple &WidthMax, const HTuple &HeightMin, const HTuple &HeightMax, const HTuple &PhiMin, const HTuple &PhiMax, const HTuple &NumRegions, const HTuple &Width, const HTuple &Height);
  static HRegionArray GenRandomRegions(const char *Type, double WidthMin, double WidthMax, double HeightMin, double HeightMax, double PhiMin, double PhiMax, long NumRegions, long Width, long Height);
  static HRegionArray GenRegionHline(const HTuple &Orientation, const HTuple &Distance);
  static HRegionArray GenRegionHline(double Orientation, double Distance);
  static HRegionArray GenRegionLine(const HTuple &BeginRow, const HTuple &BeginCol, const HTuple &EndRow, const HTuple &EndCol);
  static HRegionArray GenRegionLine(long BeginRow, long BeginCol, long EndRow, long EndCol);
  virtual HBool TestEqualRegion(const HRegionArray &Regions2) const;
  virtual HImage PaintRegion(const HImage &Image, const HTuple &Grayval, const HTuple &Type) const;
  virtual HImage PaintRegion(const HImage &Image, double Grayval, const char *Type) const;
  virtual HRegionArray MergeRegionsLineScan(const HRegionArray &PrevRegions, HRegionArray *PrevMergedRegions, const HTuple &ImageHeight, const HTuple &MergeBorder, const HTuple &MaxImagesRegion) const;
  virtual HRegionArray MergeRegionsLineScan(const HRegionArray &PrevRegions, HRegionArray *PrevMergedRegions, long ImageHeight, const char *MergeBorder, long MaxImagesRegion) const;
  virtual HRegionArray PartitionRectangle(const HTuple &Width, const HTuple &Height) const;
  virtual HRegionArray PartitionRectangle(double Width, double Height) const;
  virtual HRegionArray PartitionDynamic(const HTuple &Distance, const HTuple &Percent) const;
  virtual HRegionArray PartitionDynamic(double Distance, double Percent) const;
  virtual HImage RegionToLabel(const HTuple &Type, const HTuple &Width, const HTuple &Height) const;
  virtual HImage RegionToLabel(const char *Type, long Width, long Height) const;
  virtual HImage RegionToBin(const HTuple &ForegroundGray, const HTuple &BackgroundGray, const HTuple &Width, const HTuple &Height) const;
  virtual HImage RegionToBin(long ForegroundGray, long BackgroundGray, long Width, long Height) const;
  virtual HRegionArray Union2(const HRegionArray &Region2) const;
  // Return the union of all input regions.
  virtual HRegion Union1(void) const;
  virtual HImage DistanceTransform(const HTuple &Metric, const HTuple &Foreground, const HTuple &Width, const HTuple &Height) const;
  virtual HImage DistanceTransform(const char *Metric, const char *Foreground, long Width, long Height) const;
  virtual HRegionArray Skeleton(void) const;
  virtual HRegionArray ProjectiveTransRegion(const HTuple &HomMat2D, const HTuple &Interpolation) const;
  virtual HRegionArray ProjectiveTransRegion(double HomMat2D, const char *Interpolation) const;
  virtual HRegionArray AffineTransRegion(const HTuple &HomMat2D, const HTuple &Interpolate) const;
  virtual HRegionArray AffineTransRegion(double HomMat2D, const char *Interpolate) const;
  virtual HRegionArray MirrorRegion(const HTuple &RowColumn, const HTuple &WidthHeight) const;
  virtual HRegionArray MirrorRegion(const char *RowColumn, long WidthHeight) const;
  virtual HRegionArray ZoomRegion(const HTuple &ScaleWidth, const HTuple &ScaleHeight) const;
  virtual HRegionArray ZoomRegion(double ScaleWidth, double ScaleHeight) const;
  virtual HRegionArray MoveRegion(const HTuple &Row, const HTuple &Column) const;
  virtual HRegionArray MoveRegion(long Row, long Column) const;
  virtual HRegionArray JunctionsSkeleton(HRegionArray *JuncPoints) const;
  virtual HRegionArray Intersection(const HRegionArray &Region2) const;
  virtual HRegion Interjacent(const HTuple &Mode) const;
  virtual HRegion Interjacent(const char *Mode) const;
  virtual HRegionArray FillUp(void) const;
  virtual HRegionArray FillUpShape(const HTuple &Feature, const HTuple &Min, const HTuple &Max) const;
  virtual HRegionArray FillUpShape(const char *Feature, double Min, double Max) const;
  virtual HRegionArray ExpandRegion(const HRegion &ForbiddenArea, const HTuple &Iterations, const HTuple &Mode) const;
  virtual HRegionArray ExpandRegion(const HRegion &ForbiddenArea, long Iterations, const char *Mode) const;
  virtual HRegionArray ClipRegionRel(const HTuple &Top, const HTuple &Bottom, const HTuple &Left, const HTuple &Right) const;
  virtual HRegionArray ClipRegionRel(long Top, long Bottom, long Left, long Right) const;
  virtual HRegionArray ClipRegion(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HRegionArray ClipRegion(long Row1, long Column1, long Row2, long Column2) const;
  virtual HRegionArray RankRegion(const HTuple &Width, const HTuple &Height, const HTuple &Number) const;
  virtual HRegionArray RankRegion(long Width, long Height, long Number) const;
  virtual HRegionArray Connection(void) const;
  virtual HRegionArray SymmDifference(const HRegionArray &Region2) const;
  virtual HRegionArray Difference(const HRegionArray &Sub) const;
  virtual HRegionArray Complement(void) const;
  virtual HRegionArray BackgroundSeg(void) const;
  virtual HRegionArray HammingChangeRegion(const HTuple &Width, const HTuple &Height, const HTuple &Distance) const;
  virtual HRegionArray HammingChangeRegion(long Width, long Height, long Distance) const;
  virtual HRegionArray RemoveNoiseRegion(const HTuple &Type) const;
  virtual HRegionArray RemoveNoiseRegion(const char *Type) const;
  virtual HRegionArray ShapeTrans(const HTuple &Type) const;
  virtual HRegionArray ShapeTrans(const char *Type) const;
  virtual HRegionArray ExpandGray(const HImage &Image, const HRegion &ForbiddenArea, const HTuple &Iterations, const HTuple &Mode, const HTuple &Threshold) const;
  virtual HRegionArray ExpandGrayRef(const HImage &Image, const HRegion &ForbiddenArea, const HTuple &Iterations, const HTuple &Mode, const HTuple &RefGray, const HTuple &Threshold) const;
  // Split lines represented by one pixel wide, non-branching lines.
  virtual HTuple SplitSkeletonLines(const HTuple &MaxDistance, HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol) const;
  virtual HTuple SplitSkeletonLines(long MaxDistance, HTuple *BeginCol, HTuple *EndRow, HTuple *EndCol) const;
  virtual HRegionArray SplitSkeletonRegion(const HTuple &MaxDistance) const;
  virtual HRegionArray SplitSkeletonRegion(long MaxDistance) const;
  virtual HRegionArray EliminateRuns(const HTuple &ElimShorter, const HTuple &ElimLonger) const;
  virtual HRegionArray EliminateRuns(long ElimShorter, long ElimLonger) const;
  virtual HTuple LearnNdimNorm(const HRegionArray &Background, const HImageArray &Image, const HTuple &Metric, const HTuple &Distance, const HTuple &MinNumberPercent, HTuple *Center, HTuple *Quality) const;
  virtual HTuple LearnNdimNorm(const HRegionArray &Background, const HImageArray &Image, const char *Metric, double Distance, double MinNumberPercent, HTuple *Center, HTuple *Quality) const;
  virtual void LearnNdimBox(const HRegionArray &Background, const HImageArray &MultiChannelImage, const HTuple &ClassifHandle) const;
  virtual void LearnNdimBox(const HRegionArray &Background, const HImageArray &MultiChannelImage, long ClassifHandle) const;
  static HRegionArray ReceiveRegion(const HTuple &Socket);
  static HRegionArray ReceiveRegion(long Socket);
  virtual void SendRegion(const HTuple &Socket) const;
  virtual void SendRegion(long Socket) const;
  virtual HXLDContArray GenContourRegionXld(const HTuple &Mode) const;
  virtual HXLDContArray GenContourRegionXld(const char *Mode) const;
};

#endif
