/*****************************************************************************
 * HShapeModel.h
 ***************************************************************************** 
 *
 * Project:     Halcon/C++
 * Description: Class HShapeModel
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.4 $
 * $Date: 2003/10/02 09:59:15 $
 *
 */

#ifndef H_SHAPE_MODEL_H
#define H_SHAPE_MODEL_H

class LIntExport HShapeModel: public HRootObject {
public:
  HShapeModel(void) {id = -1;}
  HShapeModel(
          const HImage &Template, 
          const HTuple &NumLevels, 
          const HTuple &AngleStart, 
          const HTuple &AngleExtent, 
          const HTuple &AngleStep, 
          const HTuple &Optimization, 
          const HTuple &Metric, 
          const HTuple &Contrast, 
          const HTuple &MinContrast);
  HShapeModel(
          const HImage &Template, 
          long          NumLevels, 
          double        AngleStart, 
          double        AngleExtent, 
          double        AngleStep, 
          const char   *Optimization = "none", 
          const char   *Metric = "use_polarity", 
          long          Contrast = 30, 
          long          MinContrast = 10);
  HShapeModel(
          const HImage &Template, 
          const HTuple &NumLevels, 
          const HTuple &AngleStart, 
          const HTuple &AngleExtent, 
          const HTuple &AngleStep, 
          const HTuple &ScaleMin, 
          const HTuple &ScaleMax, 
          const HTuple &ScaleStep, 
          const HTuple &Optimization, 
          const HTuple &Metric, 
          const HTuple &Contrast, 
          const HTuple &MinContrast);
  HShapeModel(
          const HImage &Template, 
          long          NumLevels, 
          double        AngleStart, 
          double        AngleExtent, 
          double        AngleStep, 
          double        ScaleMin, 
          double        ScaleMax, 
          double        ScaleStep, 
          const char   *Optimization = "none", 
          const char   *Metric = "use_polarity", 
          long          Contrast = 30, 
          long          MinContrast = 10);

  ~HShapeModel(void);
private:
  // Copying and Assignment not allowed
  HShapeModel(const HShapeModel &inst);
  HShapeModel &operator= (const HShapeModel &inst);
  // Internal Cleanup
  void Cleanup(void);

public: 
  // Tools
  const char *ClassName(void) const {return "HShapeModel";}

  // Cast Operator
  operator HTuple () const {return HTuple(GetHandle());}

  // Access and Manipulation
  long GetHandle(void) const {return id;}
  void SetHandle(long ID) {id = ID;}

private:
  //Handle
  long  id;
public:
  // Read a shape model from a file.
  virtual void ReadShapeModel(const HTuple &FileName);
  virtual void ReadShapeModel(const char *FileName);
  // Write a shape model to a file.
  virtual void WriteShapeModel(const HTuple &FileName) const;
  virtual void WriteShapeModel(const char *FileName) const;
  // Return the contour representation of a shape model.
  virtual HXLDContArray GetShapeModelContours(const HTuple &Level) const;
  virtual HXLDContArray GetShapeModelContours(long Level) const;
  // Return the parameters of a shape model.
  virtual long GetShapeModelParams(HTuple *AngleStart, HTuple *AngleExtent, HTuple *AngleStep, HTuple *ScaleMin, HTuple *ScaleMax, HTuple *ScaleStep, HTuple *Metric, HTuple *MinContrast) const;
  virtual long GetShapeModelParams(double *AngleStart, double *AngleExtent, double *AngleStep, double *ScaleMin, double *ScaleMax, double *ScaleStep, char *Metric, long *MinContrast) const;
  // Return the origin (reference point) of a shape model.
  virtual double GetShapeModelOrigin(HTuple *Column) const;
  virtual double GetShapeModelOrigin(double *Column) const;
  // Set the origin (reference point) of a shape model.
  virtual void SetShapeModelOrigin(const HTuple &Row, const HTuple &Column) const;
  virtual void SetShapeModelOrigin(double Row, double Column) const;
  // Find the best matches of multiple scale invariant shape models.
  virtual HTuple FindScaledShapeModels(const HImageArray &Image, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindScaledShapeModels(const HImageArray &Image, double AngleStart, double AngleExtent, double ScaleMin, double ScaleMax, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score, HTuple *Model) const;
  // Find the best matches of multiple shape models.
  virtual HTuple FindShapeModels(const HImageArray &Image, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Score, HTuple *Model) const;
  virtual HTuple FindShapeModels(const HImageArray &Image, double AngleStart, double AngleExtent, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Score, HTuple *Model) const;
  // Find the best matches of a scale invariant shape model in an image.
  virtual HTuple FindScaledShapeModel(const HImage &Image, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score) const;
  virtual HTuple FindScaledShapeModel(const HImage &Image, double AngleStart, double AngleExtent, double ScaleMin, double ScaleMax, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Scale, HTuple *Score) const;
  // Find the best matches of a shape model in an image.
  virtual HTuple FindShapeModel(const HImage &Image, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &MinScore, const HTuple &NumMatches, const HTuple &MaxOverlap, const HTuple &SubPixel, const HTuple &NumLevels, const HTuple &Greediness, HTuple *Column, HTuple *Angle, HTuple *Score) const;
  virtual HTuple FindShapeModel(const HImage &Image, double AngleStart, double AngleExtent, double MinScore, long NumMatches, double MaxOverlap, const char *SubPixel, long NumLevels, double Greediness, HTuple *Column, HTuple *Angle, HTuple *Score) const;
  // Prepare a shape model for scale invariant matching.
  virtual void CreateScaledShapeModel(const HImage &Template, const HTuple &NumLevels, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &AngleStep, const HTuple &ScaleMin, const HTuple &ScaleMax, const HTuple &ScaleStep, const HTuple &Optimization, const HTuple &Metric, const HTuple &Contrast, const HTuple &MinContrast);
  virtual void CreateScaledShapeModel(const HImage &Template, long NumLevels, double AngleStart, double AngleExtent, double AngleStep, double ScaleMin, double ScaleMax, double ScaleStep, const char *Optimization, const char *Metric, long Contrast, long MinContrast);
  // Prepare a shape model for matching.
  virtual void CreateShapeModel(const HImage &Template, const HTuple &NumLevels, const HTuple &AngleStart, const HTuple &AngleExtent, const HTuple &AngleStep, const HTuple &Optimization, const HTuple &Metric, const HTuple &Contrast, const HTuple &MinContrast);
  virtual void CreateShapeModel(const HImage &Template, long NumLevels, double AngleStart, double AngleExtent, double AngleStep, const char *Optimization, const char *Metric, long Contrast, long MinContrast);
};

#endif
