/*****************************************************************************
 * HXLDArray.h
 ***************************************************************************** 
 *
 * Project:     Halcon/libhalcon
 * Description: Class HXLDArray
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 */

#ifndef XLD_ARRAY_H
#define XLD_ARRAY_H


/****************************************************************************/
/*                               HXLDArray                                  */
/****************************************************************************/

class LIntExport HXLDArray: public HObjectArray {
public:
  HXLDArray(void) : HObjectArray(), array(NULL) {num_xld_arrays++;}
  HXLDArray(long n) : HObjectArray(n), array(NULL) {num_xld_arrays++;}
  HXLDArray(HBaseArrayMode m) : HObjectArray(m), 
                                array(NULL) {num_xld_arrays++;}
  HXLDArray(Hkey *key, long num);
  HXLDArray(const Hobject &obj);
  HXLDArray(const HXLD &xld);
  HXLDArray(const HXLDArray &arr);
  ~HXLDArray(void);

  HXLDArray &operator= (const HXLDArray &arr);

  // Tools
  virtual const char  *HClassName(void)     const;
  virtual const char  *ClassName(void)      const;
  virtual const char  *InstClassName(void)  const;
  virtual       Hkey   Key(long i)          const {return array[i].Key();}
  virtual    Hobject   ElemId(long i)       const {return array[i].Id();}

  virtual void         Display(const HWindow &w) const;

  // Access and Manipulation
  HXLD  const     &operator[] (long index) const;
  HXLD            &operator[] (long index);
  HXLDArray       operator() (long min, long max) const;
  HXLDArray       &Append(const HXLD &xld);
  HXLDArray       &Append(const HXLDArray &xlds);
  virtual void    SetKeys(Hkey *key, long num);
  void            CopyHXLDArray(const HXLDArray &in);
  virtual void    DeleteArray(void);
  // Accessing array elements without index checking -> fast AND risky
  HXLD            Elem(long index) const {return array[index];}

  // Class Functions
  static long NumInst(void) {return num_xld_arrays;}

protected:
  HXLD          *array;
private:
  static long   num_xld_arrays; 
  void          NewLength(long new_num, long old_num);

public:
  virtual HXLDArray SelectXldPoint(const HTuple &Row, const HTuple &Column) const;
  virtual HXLDArray SelectXldPoint(double Row, double Column) const;
  virtual HTuple TestXldPoint(const HTuple &Row, const HTuple &Column) const;
  virtual HTuple TestXldPoint(double Row, double Column) const;
  // Select contours using shape features.
  virtual HXLDArray SelectShapeXld(const HTuple &Features, const HTuple &Operation, const HTuple &Min, const HTuple &Max) const;
  virtual HXLDArray SelectShapeXld(const char *Features, const char *Operation, double Min, double Max) const;
  virtual HTuple OrientationXld(void) const;
  virtual HTuple EccentricityXld(HTuple *Bulkiness, HTuple *StructureFactor) const;
  virtual HTuple CompactnessXld(void) const;
  virtual HTuple DiameterXld(HTuple *Column1, HTuple *Row2, HTuple *Column2, HTuple *Diameter) const;
  virtual HTuple ConvexityXld(void) const;
  virtual HTuple CircularityXld(void) const;
  virtual HTuple EllipticAxisXld(HTuple *Rb, HTuple *Phi) const;
  virtual HTuple SmallestRectangle2Xld(HTuple *Column, HTuple *Phi, HTuple *Length1, HTuple *Length2) const;
  virtual HTuple SmallestRectangle1Xld(HTuple *Column1, HTuple *Row2, HTuple *Column2) const;
  virtual HTuple SmallestCircleXld(HTuple *Column, HTuple *Radius) const;
  virtual HXLDArray ShapeTransXld(const HTuple &Type) const;
  virtual HXLDArray ShapeTransXld(const char *Type) const;
  virtual HTuple LengthXld(void) const;
  virtual HTuple MomentsAnyXld(const HTuple &Mode, const HTuple &PointOrder, const HTuple &Area, const HTuple &CenterRow, const HTuple &CenterCol, const HTuple &P, const HTuple &Q) const;
  virtual HTuple MomentsAnyXld(const char *Mode, const char *PointOrder, double Area, double CenterRow, double CenterCol, long P, long Q) const;
  virtual HTuple MomentsXld(HTuple *M20, HTuple *M02) const;
  virtual HTuple AreaCenterXld(HTuple *Row, HTuple *Column, HTuple *PointOrder) const;
  virtual HImage PaintXld(const HImage &Image, const HTuple &Grayval) const;
  virtual HImage PaintXld(const HImage &Image, double Grayval) const;
  static HXLDArray ReceiveXld(const HTuple &Socket);
  static HXLDArray ReceiveXld(long Socket);
  virtual void SendXld(const HTuple &Socket) const;
  virtual void SendXld(long Socket) const;
  // Display an XLD object.
  virtual void DispXld(const HTuple &WindowHandle) const;
  virtual void DispXld(long WindowHandle) const;
};

#endif
