/*****************************************************************************
 * HXLDContArray.h
 ***************************************************************************** 
 *
 * Project:     Halcon/libhalcon
 * Description: Class HXLDContArray
 *
 * (c) 1996-2001 by MVTec Software GmbH
 *                  Neherstr. 1, 81675 Muenchen, Germany
 * 
 *****************************************************************************
 *
 * $Revision: 1.13 $
 * $Date: 2001/11/22 13:46:55 $
 *
 */

#ifndef XLD_CONT_ARRAY_H
#define XLD_CONT_ARRAY_H


/****************************************************************************/
/*                               HXLDContArray                              */
/****************************************************************************/

class LIntExport HXLDContArray: public HXLDArray {
public:
  HXLDContArray(void) : HXLDArray() {num_xld_cont_arrays++;}
  HXLDContArray(Hkey *key, long num);
  HXLDContArray(const Hobject &obj);
  HXLDContArray(const HXLDCont &cont);
  HXLDContArray(const HXLD &xld);
  HXLDContArray(const HXLDContArray &arr);
  HXLDContArray(const HXLDArray &arr);
  ~HXLDContArray(void) {num_xld_cont_arrays--;}

  HXLDContArray &operator= (const HXLDContArray &arr);
  HXLDContArray &operator= (const HXLDArray &arr);

  // Tools
  virtual const char *HClassName(void)    const;
  virtual const char *ClassName(void)     const;
  virtual const char *InstClassName(void) const;
  virtual       INT4  ClassDBID(void)     const {return XLD_CONTOUR_ID;}
  virtual       Hkey  Key(long i)         const {return array[i].Key();}
  virtual    Hobject  ElemId(long i)      const {return array[i].Id();}

  // Access and Manipulation
  HXLDCont  const   &operator[] (long index) const;
  HXLDCont          &operator[] (long index);
  HXLDContArray      operator() (long min, long max) const;
  HXLDContArray     &Append(const HXLDCont &cont);
  HXLDContArray     &Append(const HXLD &xld);
  HXLDContArray     &Append(const HXLDContArray &conts);
  HXLDContArray     &Append(const HXLDArray &xlds);
  virtual void       SetKeys(Hkey *key, long num);
  void               CopyHXLDContArray(const HXLDContArray &in);
  void               CopyHXLDArray(const HXLDArray &in);

  // Class Functions
  static long NumInst(void) {return num_xld_cont_arrays;}

private:
  static long   num_xld_cont_arrays; 
  void          NewLength(long new_num, long old_num);

public:
  virtual HTuple DistanceCcMin(const HXLDContArray &Contour2, const HTuple &Mode) const;
  virtual HTuple DistanceCcMin(const HXLDContArray &Contour2, const char *Mode) const;
  virtual HTuple DistanceCc(const HXLDContArray &Contour2, const HTuple &Mode, HTuple *DistanceMax) const;
  virtual HTuple DistanceCc(const HXLDContArray &Contour2, const char *Mode, HTuple *DistanceMax) const;
  virtual HXLDContArray ChangeRadialDistortionContoursXld(const HTuple &CamParIn, const HTuple &CamParOut) const;
  virtual HXLDContArray ChangeRadialDistortionContoursXld(double CamParIn, double CamParOut) const;
  // Generate one XLD contour in the shape of a cross for each input point.
  static HXLDContArray GenCrossContourXld(const HTuple &Row, const HTuple &Col, const HTuple &Size, const HTuple &Angle);
  static HXLDContArray GenCrossContourXld(double Row, double Col, double Size, double Angle);
  // Sorting of contours with respect to their relative position.
  virtual HXLDContArray SortContoursXld(const HTuple &SortMode, const HTuple &Order, const HTuple &RowOrCol) const;
  virtual HXLDContArray SortContoursXld(const char *SortMode, const char *Order, const char *RowOrCol) const;
  virtual HXLDContArray MergeContLineScanXld(const HXLDContArray &PrevConts, HXLDContArray *PrevMergedConts, const HTuple &ImageHeight, const HTuple &Margin, const HTuple &MergeBorder, const HTuple &MaxImagesCont) const;
  virtual HXLDContArray MergeContLineScanXld(const HXLDContArray &PrevConts, HXLDContArray *PrevMergedConts, long ImageHeight, double Margin, const char *MergeBorder, long MaxImagesCont) const;
  static HXLDContArray ReadContourXldArcInfo(const HTuple &FileName);
  static HXLDContArray ReadContourXldArcInfo(const char *FileName);
  virtual void WriteContourXldArcInfo(const HTuple &FileName) const;
  virtual void WriteContourXldArcInfo(const char *FileName) const;
  // Compute the parallel contour of an XLD contour.
  virtual HXLDContArray GenParallelContourXld(const HTuple &Mode, const HTuple &Distance) const;
  virtual HXLDContArray GenParallelContourXld(const char *Mode, double Distance) const;
  virtual HXLDContArray SegmentContoursXld(const HTuple &Mode, const HTuple &SmoothCont, const HTuple &MaxLineDist1, const HTuple &MaxLineDist2) const;
  virtual HXLDContArray SegmentContoursXld(const char *Mode, long SmoothCont, double MaxLineDist1, double MaxLineDist2) const;
  virtual HTuple FitCircleContourXld(const HTuple &Algorithm, const HTuple &MaxNumPoints, const HTuple &MaxClosureDist, const HTuple &ClippingEndPoints, const HTuple &Iterations, const HTuple &ClippingFactor, HTuple *Column, HTuple *Radius, HTuple *StartPhi, HTuple *EndPhi, HTuple *PointOrder) const;
  virtual HTuple FitCircleContourXld(const char *Algorithm, long MaxNumPoints, double MaxClosureDist, long ClippingEndPoints, long Iterations, double ClippingFactor, HTuple *Column, HTuple *Radius, HTuple *StartPhi, HTuple *EndPhi, HTuple *PointOrder) const;
  virtual HTuple FitLineContourXld(const HTuple &Algorithm, const HTuple &MaxNumPoints, const HTuple &ClippingEndPoints, const HTuple &Iterations, const HTuple &ClippingFactor, HTuple *ColBegin, HTuple *RowEnd, HTuple *ColEnd, HTuple *Nr, HTuple *Nc, HTuple *Dist) const;
  virtual HTuple FitLineContourXld(const char *Algorithm, long MaxNumPoints, long ClippingEndPoints, long Iterations, double ClippingFactor, HTuple *ColBegin, HTuple *RowEnd, HTuple *ColEnd, HTuple *Nr, HTuple *Nc, HTuple *Dist) const;
  virtual HTuple DistEllipseContourXld(const HTuple &Mode, const HTuple &MaxNumPoints, const HTuple &ClippingEndPoints, const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2, HTuple *MaxDist, HTuple *AvgDist, HTuple *SigmaDist) const;
  virtual HTuple DistEllipseContourXld(const char *Mode, long MaxNumPoints, long ClippingEndPoints, double Row, double Column, double Phi, double Radius1, double Radius2, HTuple *MaxDist, HTuple *AvgDist, HTuple *SigmaDist) const;
  virtual HTuple FitEllipseContourXld(const HTuple &Algorithm, const HTuple &MaxNumPoints, const HTuple &MaxClosureDist, const HTuple &ClippingEndPoints, const HTuple &VossTabSize, const HTuple &Iterations, const HTuple &ClippingFactor, HTuple *Column, HTuple *Phi, HTuple *Radius1, HTuple *Radius2, HTuple *StartPhi, HTuple *EndPhi, HTuple *PointOrder) const;
  virtual HTuple FitEllipseContourXld(const char *Algorithm, long MaxNumPoints, double MaxClosureDist, long ClippingEndPoints, long VossTabSize, long Iterations, double ClippingFactor, HTuple *Column, HTuple *Phi, HTuple *Radius1, HTuple *Radius2, HTuple *StartPhi, HTuple *EndPhi, HTuple *PointOrder) const;
  static HXLDContArray GenEllipseContourXld(const HTuple &Row, const HTuple &Column, const HTuple &Phi, const HTuple &Radius1, const HTuple &Radius2, const HTuple &StartPhi, const HTuple &EndPhi, const HTuple &PointOrder, const HTuple &Resolution);
  static HXLDContArray GenEllipseContourXld(double Row, double Column, double Phi, double Radius1, double Radius2, double StartPhi, double EndPhi, const char *PointOrder, double Resolution);
  virtual HXLDContArray AddNoiseWhiteContourXld(const HTuple &NumRegrPoints, const HTuple &Amp) const;
  virtual HXLDContArray AddNoiseWhiteContourXld(long NumRegrPoints, double Amp) const;
  // Approximate XLD contours by polygons.
  virtual HXLDPolyArray GenPolygonsXld(const HTuple &Type, const HTuple &Alpha) const;
  virtual HXLDPolyArray GenPolygonsXld(const char *Type, double Alpha) const;
  virtual HXLDContArray ProjectiveTransContourXld(const HTuple &HomMat2D) const;
  virtual HXLDContArray ProjectiveTransContourXld(double HomMat2D) const;
  virtual HXLDContArray AffineTransContourXld(const HTuple &HomMat2D) const;
  virtual HXLDContArray AffineTransContourXld(double HomMat2D) const;
  // Close an XLD contour.
  virtual HXLDContArray CloseContoursXld(void) const;
  // Clip an XLD contour.
  virtual HXLDContArray ClipContoursXld(const HTuple &Row1, const HTuple &Column1, const HTuple &Row2, const HTuple &Column2) const;
  virtual HXLDContArray ClipContoursXld(long Row1, long Column1, long Row2, long Column2) const;
  // Select XLD contours with a local maximum of gray values.
  virtual HXLDContArray LocalMaxContoursXld(const HImage &Image, const HTuple &MinPercent, const HTuple &MinDiff, const HTuple &Distance) const;
  virtual HXLDContArray LocalMaxContoursXld(const HImage &Image, long MinPercent, long MinDiff, long Distance) const;
  // Merge neighboring straight contours having a similar distance from a 
  // given line.
  virtual HXLDContArray UnionStraightContoursHistoXld(HXLDContArray *SelectedContours, const HTuple &RefLineStartRow, const HTuple &RefLineStartColumn, const HTuple &RefLineEndRow, const HTuple &RefLineEndColumn, const HTuple &Width, const HTuple &MaxWidth, const HTuple &FilterSize, HTuple *HistoValues) const;
  virtual HXLDContArray UnionStraightContoursHistoXld(HXLDContArray *SelectedContours, long RefLineStartRow, long RefLineStartColumn, long RefLineEndRow, long RefLineEndColumn, long Width, long MaxWidth, long FilterSize, HTuple *HistoValues) const;
  // Merge neighboring straight contours having a similar direction.
  virtual HXLDContArray UnionStraightContoursXld(const HTuple &MaxDist, const HTuple &MaxDiff, const HTuple &Percent, const HTuple &Mode, const HTuple &Iterations) const;
  // Union of collinear contours (operator with extended functionality).
  virtual HXLDContArray UnionCollinearContoursExtXld(const HTuple &MaxDistAbs, const HTuple &MaxDistRel, const HTuple &MaxShift, const HTuple &MaxAngle, const HTuple &MaxOverlap, const HTuple &MaxRegrError, const HTuple &MaxCosts, const HTuple &WeightDist, const HTuple &WeightShift, const HTuple &WeightAngle, const HTuple &WeightLink, const HTuple &WeightRegr, const HTuple &Mode) const;
  virtual HXLDContArray UnionCollinearContoursExtXld(double MaxDistAbs, double MaxDistRel, double MaxShift, double MaxAngle, double MaxOverlap, double MaxRegrError, double MaxCosts, double WeightDist, double WeightShift, double WeightAngle, double WeightLink, double WeightRegr, const char *Mode) const;
  // Union of collinear contours.
  virtual HXLDContArray UnionCollinearContoursXld(const HTuple &MaxDistAbs, const HTuple &MaxDistRel, const HTuple &MaxShift, const HTuple &MaxAngle, const HTuple &Mode) const;
  virtual HXLDContArray UnionCollinearContoursXld(double MaxDistAbs, double MaxDistRel, double MaxShift, double MaxAngle, const char *Mode) const;
  // Select XLD contours according to several features.
  virtual HXLDContArray SelectContoursXld(const HTuple &Feature, const HTuple &Min1, const HTuple &Max1, const HTuple &Min2, const HTuple &Max2) const;
  virtual HXLDContArray SelectContoursXld(const char *Feature, double Min1, double Max1, double Min2, double Max2) const;
  // Return XLD contour parameters.
  virtual HTuple GetRegressParamsXld(HTuple *Nx, HTuple *Ny, HTuple *Dist, HTuple *Fpx, HTuple *Fpy, HTuple *Lpx, HTuple *Lpy, HTuple *Mean, HTuple *Deviation) const;
  // Calculate the parameters of a regression line to an XLD contour.
  virtual HXLDContArray RegressContoursXld(const HTuple &Mode, const HTuple &Iterations) const;
  virtual HXLDContArray RegressContoursXld(const char *Mode, long Iterations) const;
  // Smooth an XLD contour.
  virtual HXLDContArray SmoothContoursXld(const HTuple &NumRegrPoints) const;
  virtual HXLDContArray SmoothContoursXld(long NumRegrPoints) const;
};

#endif
